@@ -39,7 +39,8 @@ PLACEMENT_API_WITH_NESTED_RESOURCES = 'placement 1.14'
 PLACEMENT_API_RETURN_PROVIDER_BODY = 'placement 1.20'
 PLACEMENT_API_ERROR_CODE = 'placement 1.23'
 PLACEMENT_API_CONSUMER_GENERATION = 'placement 1.28'
-PLACEMENT_API_LATEST_SUPPORTED = PLACEMENT_API_CONSUMER_GENERATION
+PLACEMENT_API_UPDATE_REPARENT = 'placement 1.37'
+PLACEMENT_API_LATEST_SUPPORTED = PLACEMENT_API_UPDATE_REPARENT
 GENERATION_CONFLICT_RETRIES = 10
 
 
@@ -64,8 +65,7 @@ def _check_placement_api_available(f):
                 # to avoid losing information.
                 raise n_exc.PlacementClientError(
                     msg=exc.response.text.replace('\n', ' '))
-            else:
-                raise
+            raise
     return wrapper
 
 
@@ -79,7 +79,7 @@ class UUIDEncoder(jsonutils.JSONEncoder):
     def default(self, o):
         if isinstance(o, uuid.UUID):
             return str(o)
-        return super(UUIDEncoder, self).default(o)
+        return super().default(o)
 
 
 class NoAuthClient(object):
@@ -233,8 +233,7 @@ class PlacementAPIClient(object):
                 if e.response.json()[
                         'errors'][0]['code'] == 'placement.concurrent_update':
                     continue
-                else:
-                    raise
+                raise
 
         raise n_exc.PlacementResourceProviderGenerationConflict(
             resource_provider=resource_provider_uuid,
@@ -277,6 +276,7 @@ class PlacementAPIClient(object):
                                                         the same name.
         :returns: The updated resource provider.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s' % resource_provider['uuid']
         # update does not tolerate if the uuid is repeated in the body
         update_body = resource_provider.copy()
@@ -325,6 +325,7 @@ class PlacementAPIClient(object):
         :raises PlacementResourceProviderNotFound: For failure to find resource
         :returns: The Resource Provider matching the UUID.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s' % resource_provider_uuid
         try:
             return self._get(url).json()
@@ -407,6 +408,7 @@ class PlacementAPIClient(object):
                                                              server side.
         :returns: The updated set of inventory records.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/inventories' % resource_provider_uuid
         body = {
             'resource_provider_generation': resource_provider_generation,
@@ -438,8 +440,7 @@ class PlacementAPIClient(object):
             if "No resource provider with uuid" in e.details:
                 raise n_exc.PlacementResourceProviderNotFound(
                     resource_provider=resource_provider_uuid)
-            else:
-                raise
+            raise
 
     @_check_placement_api_available
     def delete_resource_provider_inventory(self, resource_provider_uuid,
@@ -461,12 +462,11 @@ class PlacementAPIClient(object):
             if "No resource provider with uuid" in e.details:
                 raise n_exc.PlacementResourceProviderNotFound(
                     resource_provider=resource_provider_uuid)
-            elif "No inventory of class" in e.details:
+            if "No inventory of class" in e.details:
                 raise n_exc.PlacementInventoryNotFound(
                     resource_provider=resource_provider_uuid,
                     resource_class=resource_class)
-            else:
-                raise
+            raise
 
     @_check_placement_api_available
     def get_inventory(self, resource_provider_uuid, resource_class):
@@ -489,12 +489,11 @@ class PlacementAPIClient(object):
             if "No resource provider with uuid" in e.details:
                 raise n_exc.PlacementResourceProviderNotFound(
                     resource_provider=resource_provider_uuid)
-            elif _("No inventory of class") in e.details:
+            if _("No inventory of class") in e.details:
                 raise n_exc.PlacementInventoryNotFound(
                     resource_provider=resource_provider_uuid,
                     resource_class=resource_class)
-            else:
-                raise
+            raise
 
     @_check_placement_api_available
     def update_resource_provider_inventory(
@@ -549,6 +548,7 @@ class PlacementAPIClient(object):
         :returns: The list of aggregates together with the resource provider
                   generation.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/aggregates' % resource_provider_uuid
         try:
             return self._get(url).json()
@@ -570,6 +570,7 @@ class PlacementAPIClient(object):
         :raises PlacementTraitNotFound: If the trait name not found.
         :returns: Evaluates to True if the trait exists.
         """
+        # pylint: disable=raise-missing-from
         url = '/traits/%s' % name
         try:
             return self._get(url)
@@ -594,6 +595,7 @@ class PlacementAPIClient(object):
         :raises PlacementTraitNotFound: If the trait did not exist.
         :returns: None.
         """
+        # pylint: disable=raise-missing-from
         url = '/traits/%s' % (name)
         try:
             self._delete(url)
@@ -626,6 +628,7 @@ class PlacementAPIClient(object):
         :returns: The new traits of the resource provider together with the
                   resource provider generation.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         body = {
             'resource_provider_generation': resource_provider_generation,
@@ -653,6 +656,7 @@ class PlacementAPIClient(object):
         :returns: The associated traits of the resource provider together
                   with the resource provider generation.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         try:
             return self._get(url).json()
@@ -670,6 +674,7 @@ class PlacementAPIClient(object):
                                                    is not found.
         :returns: None.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         try:
             self._delete(url)
@@ -692,6 +697,7 @@ class PlacementAPIClient(object):
                                                 is not found.
         :returns: The name of resource class and its set of links.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_classes/%s' % (name)
         try:
             return self._get(url).json()
@@ -728,6 +734,7 @@ class PlacementAPIClient(object):
                                                 is not found.
         :returns: None.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_classes/%s' % (name)
         try:
             self._delete(url)
@@ -768,7 +775,16 @@ class PlacementAPIClient(object):
             for drctn, min_kbps_diff in minbw_alloc_diff.items():
                 orig_kbps = body['allocations'][rp_uuid]['resources'][drctn]
                 new_kbps = orig_kbps + min_kbps_diff
-                body['allocations'][rp_uuid]['resources'][drctn] = new_kbps
+                if new_kbps > 0:
+                    body['allocations'][rp_uuid]['resources'][drctn] = new_kbps
+                else:
+                    # Remove the resource class if the new value for min_kbps
+                    # is 0
+                    resources = body['allocations'][rp_uuid]['resources']
+                    if len(resources) > 1:
+                        resources.pop(drctn, None)
+                    else:
+                        body['allocations'].pop(rp_uuid)
             try:
                 # Update allocations has no return body, but leave the loop
                 return self.update_allocation(consumer_uuid, body)
@@ -776,8 +792,7 @@ class PlacementAPIClient(object):
                 resp = e.response.json()
                 if resp['errors'][0]['code'] == 'placement.concurrent_update':
                     continue
-                else:
-                    raise
+                raise
         raise n_exc.PlacementAllocationGenerationConflict(
             consumer=consumer_uuid)
 
